"""
Tests common to sets
"""
import copy

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class PassThru(Exception):
    pass


def check_pass_thru():
    raise PassThru
    yield 1


class SetTestBase:

    def setUp(self):
        self.word = word = 'simsalabim'
        self.otherword = 'madagascar'
        self.letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        self.s = self.type2test(word)
        self.d = dict.fromkeys(word)

    def test_uniquification(self):
        actual = sorted(self.s)
        expected = sorted(self.d)
        self.assertEqual(actual, expected)
        self.assertRaises(PassThru, self.type2test, check_pass_thru())
        self.assertRaises(TypeError, self.type2test, [[]])

    def test_len(self):
        self.assertEqual(len(self.s), len(self.d))

    def test_contains(self):
        for c in self.letters:
            self.assertEqual(c in self.s, c in self.d)
        self.assertRaises(TypeError, self.s.__contains__, [[]])

    def test_isdisjoint(self):
        def f(s1, s2):
            'Pure python equivalent of isdisjoint()'
            return not set(s1).intersection(s2)
        for larg in '', 'a', 'ab', 'abc', 'ababac', 'cdc', 'cc', 'efgfe', 'ccb', 'ef':
            s1 = self.type2test(larg)
            for rarg in '', 'a', 'ab', 'abc', 'ababac', 'cdc', 'cc', 'efgfe', 'ccb', 'ef':
                for C in set, frozenset, dict.fromkeys, str, list, tuple:
                    s2 = C(rarg)
                    actual = s1.isdisjoint(s2)
                    expected = f(s1, s2)
                    self.assertEqual(actual, expected)
                    self.assertTrue(actual is True or actual is False)

    def test_or(self):
        i = self.type2test(set(self.word).union(self.otherword))
        self.assertEqual(self.s | self.type2test(self.otherword), i)
        self.assertEqual(self.s | self.otherword, i)

    def test_and(self):
        i = self.type2test(set(self.word).intersection(self.otherword))
        self.assertEqual(self.s & self.type2test(self.otherword), i)
        self.assertEqual(self.s & self.otherword, i)

    def test_sub(self):
        i = self.type2test(set(self.word).difference(self.otherword))
        self.assertEqual(self.s - self.type2test(self.otherword), i)
        self.assertEqual(self.s - self.otherword, i)

    def test_xor(self):
        i = self.type2test(set(self.word).symmetric_difference(self.otherword))
        self.assertEqual(self.s ^ self.type2test(self.otherword), i)
        self.assertEqual(self.s ^ self.otherword, i)

    def test_equality(self):
        self.assertEqual(self.s, self.type2test(self.word))
        self.assertNotEqual(self.s, self.type2test(self.otherword))

    def test_setOfFrozensets(self):
        t = map(frozenset, ['abcdef', 'bcd', 'bdcb', 'fed', 'fedccba'])
        s = self.type2test(t)
        self.assertEqual(len(s), 3)

    def test_sub_and_super(self):
        p, q, r = map(self.type2test, ['ab', 'abcde', 'def'])
        self.assertTrue(p < q)
        self.assertTrue(p <= q)
        self.assertTrue(q <= q)
        self.assertTrue(q > p)
        self.assertTrue(q >= p)
        self.assertFalse(q < r)
        self.assertFalse(q <= r)
        self.assertFalse(q > r)
        self.assertFalse(q >= r)
        self.assertTrue(set('a').issubset('abc'))
        self.assertTrue(set('abc').issuperset('a'))
        self.assertFalse(set('a').issubset('cbs'))
        self.assertFalse(set('cbs').issuperset('a'))

    def test_deepcopy(self):
        class Tracer:
            def __init__(self, value):
                self.value = value
            def __hash__(self):
                return self.value
            def __deepcopy__(self, memo=None):
                return Tracer(self.value + 1)
        t = Tracer(10)
        s = self.type2test([t])
        dup = copy.deepcopy(s)
        self.assertNotEqual(id(s), id(dup))
        for elem in dup:
            newt = elem
        self.assertNotEqual(id(t), id(newt))
        self.assertEqual(t.value + 1, newt.value)



import csv

class Agent:
    def __init__(self, code, first_name, last_name, rank, active, peer, *args):
        """
        An agent that is used for declarative operations      
        :param code: The agent id
        :param first_name: first name
        :param last_name: last name
        :param rank: rank in the system
        :param active: is the agent active
        :param peer: peer agents
        :param args: countries where the agent can be active
        """
        self.code = code
        self.first_name = first_name
        self.last_name = last_name
        self.rank = int(rank)
        self.active = True if active.strip() == 'TRUE' else False
        if peer:
            self.peers = peer.split('~')
        else:
            self.peers = []
        self.visited = list(args)

    def __str__(self):
        return self.code

    def __repr__(self):
        return self.code

people_csv = [
    "730-46-0957,Cass,      Lamba,       91,  TRUE,         ,Sweden,Switzerland,China,Georgia,",
    "186-01-5810,Barbara,   Rosewell,   487,  TRUE,         ,Indonesia,Poland,Albania,Portugal,",
    "424-16-0664,Elnore,    Dillestone,  95,  TRUE,730-46-0957,Ireland,Russia,Philippines,Japan,",
    "694-68-6118,Brig,      Derham,     367, FALSE,186-01-5810,Pakistan,Russia,China,Papua New Guinea,",
    "212-70-6483,Adelbert,  Michelet,   166, FALSE,186-01-5810~424-16-0664,Honduras,Spain,Brazil,Sierra Leone,",
    "824-34-4142,Aldon,     Craske,     437, FALSE,,Italy,Poland,Philippines,Argentina,",
    "539-35-1184,Valentine, Woolvin,     13, FALSE,694-68-6118,Spain,Bulgaria,Indonesia,,",
    "861-26-2185,Godard,    Gadie,       99, FALSE,212-70-6483,Portugal,Brazil,Thailand,China,",
    "368-95-4835,Etan,      Bumphries,  291, FALSE,         ,Tunisia,France,Mexico,Armenia,",
    "859-05-6244,Sutherlan, McElwee,    434,  TRUE,824-34-4142,China,Russia,Bosnia and Herzegovina,China,"]

agents = dict()
agentreader = csv.reader(people_csv, delimiter=',')
for row in agentreader:
    p = Agent(*row)
    agents[p.code] = p

agents_get = agents.get
for k, v in agents.items():
    v.peers = [x for x in [agents_get(p) for p in v.peers] if x is not None]


class MochilaTest:

    def test_aggregate(self):
        def get_name(p):
            return "{} {}".format(p.first_name, p.last_name)

        def get_code(p):
            return p.code

        mochila = self.type2test(agents.values())
        d = mochila.aggregate(get_code, get_name)
        for code, p in agents.items():
            name = d[code]
            self.assertIn(p.first_name, name)
            self.assertIn(p.last_name, name)

    def test_closure(self):
        def get_peer(agent):
            return agent.peers

        mochila = self.type2test(agents.values())
        cl = mochila.closure(get_peer)
        self.assertEqual(10, len(cl))

        def flatten(iterable):
            if isinstance(iterable, Agent):
                yield iterable.code
            else:
                for x in iterable:
                    if isinstance(x, Agent):
                        yield x.code
                    elif x:
                        for x2 in x:
                            yield from flatten(x2)

        result = []
        for peerh in cl:
            agent_peers = list(flatten(peerh))
            if agent_peers:
                result.append(agent_peers)
        self.assertEqual(6, len(result))
        # FIXME Can we do more proper tests?

    def test_collect(self):

        def get_countries(agent):
            return [c for c in agent.visited if len(c) > 0]  # Ignore empty

        mochila = self.type2test(agents.values())
        a_countries = mochila.collect(get_countries).flatten()
        countries = {"Sweden", "Switzerland", "China", "Georgia", "Indonesia", "Poland", "Albania", "Portugal",
                     "Ireland", "Russia" "Philippines", "Japan", "Pakistan", "Russia", "China", "Papua New Guinea",
                     "Honduras", "Spain", "Brazil", "Sierra Leone", "Italy", "Poland", "Philippines", "Argentina",
                     "Spain", "Bulgaria", "Indonesia", "Portugal", "Brazil", "Thailand", "China", "Tunisia", "France",
                     "Mexico", "Armenia", "China", "Russia", "Bosnia and Herzegovina", "China"}
        for ac in a_countries:
            self.assertIn(ac, countries)

        # oc = a_countries.asOrderedSet()
        # import pprint
        # pprint.pprint(oc)

    def test_exists(self):

        def high_rank(agent):
            return agent.rank <= 50

        mochila = self.type2test(agents.values())
        self.assertTrue(mochila.exists(high_rank))

    def test_for_all(self):

        def at_least_one_country(agent):
            return len(agent.visited) > 0

        mochila = self.type2test(agents.values())
        self.assertTrue(mochila.for_all(at_least_one_country))

    def test_one(self):

        def find_539_35_1184(agent):
            return agent.code == "539-35-1184"

        mochila = self.type2test(agents.values())
        self.assertTrue(mochila.one(find_539_35_1184))

    def test_rejet(self):

        def active(agent):
            return agent.active

        mochila = self.type2test(agents.values())
        inactive = mochila.reject(active)
        inactive_codes = {"694-68-6118", "212-70-6483", "824-34-4142", "539-35-1184", "861-26-2185", "368-95-4835"}
        for ic in inactive_codes:
            agent = agents[ic]
            self.assertIn(agent, inactive)
        self.assertEqual(6, len(inactive))

    def test_select(self):

        def active(agent):
            return agent.active

        mochila = self.type2test(agents.values())
        active = mochila.select(active)
        active_codes = {"730-46-0957", "186-01-5810", "424-16-0664", "859-05-6244"}
        for ic in active_codes:
            agent = agents[ic]
            self.assertIn(agent, active)
        self.assertEqual(4, len(active))

    def test_select_one(self):

        def active(agent):
            return agent.active

        mochila = self.type2test(agents.values())
        active = mochila.select_one(active)
        active_codes = {"730-46-0957", "186-01-5810", "424-16-0664", "859-05-6244"}
        self.assertIn(active, [agents[code] for code in active_codes])

    def test_sort_by(self):

        def by_rank(agent):
            return agent.rank

        mochila = self.type2test(agents.values())
        sorted = mochila.sort_by(by_rank)
        ordered_ranks = ["539-35-1184", "730-46-0957", "424-16-0664", "861-26-2185", "212-70-6483", "368-95-4835",
                         "694-68-6118", "859-05-6244", "824-34-4142", "186-01-5810", ]
        for i, code in enumerate(ordered_ranks):
            agent = agents[code]
            self.assertEqual(agent, sorted[i])



from collections import Counter
import itertools
import unittest
import weakref
import sys

import mochila
from test import test_set_base
from test import test_mochila

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class MultiSetTest(test_set_base.SetTestBase, test_mochila.MochilaTest, unittest.TestCase):

    type2test = mochila.MultiSet
    basetype = mochila.MultiSet

    def test_init(self):
        # Set like init
        s = self.type2test()
        s.__init__(self.word)
        w = set()
        for c in self.word:
            if c not in w:
                w.add(c)
        self.assertEqual(s._items.keys(), w)
        s.__init__(self.otherword)
        w = set()
        for c in self.otherword:
            if c not in w:
                w.add(c)
        self.assertEqual(s._items.keys(), w)
        vals = {'a': 4, 'b': 2}
        s = self.type2test(vals)
        self.assertIn('a', s)
        self.assertIn('b', s)
        self.assertEqual(4, s['a'])
        self.assertEqual(2, s['b'])
        s = self.type2test(a=3, b=5)
        self.assertIn('a', s)
        self.assertIn('b', s)
        self.assertEqual(3, s['a'])
        self.assertEqual(5, s['b'])
        l = self.type2test()
        self.assertRaises(TypeError, l.__init__, (), 2)

    def test_uniquification(self):
        actual = sorted(self.s)
        expected = sorted(self.word)
        self.assertEqual(actual, expected)
        self.assertRaises(test_set_base.PassThru, self.type2test, test_set_base.check_pass_thru())
        self.assertRaises(TypeError, self.type2test, [[]])

    def test_len(self):
        d = Counter(self.word)
        total = sum(d.values())
        self.assertEqual(len(self.s), total)

    def test_addition(self):
        s2 = self.type2test(self.otherword)
        st = self.s + s2
        oracle = self.type2test(Counter(self.word) + Counter(self.otherword))
        self.assertEqual(st, oracle)

    def test_iadd(self):
        s2 = self.type2test(self.otherword)
        self.s += s2
        oracle = self.type2test(Counter(self.word) + Counter(self.otherword))
        self.assertEqual(self.s, oracle)

    def test_sub(self):
        s2 = self.type2test(self.otherword)
        st = self.s - s2
        oracle = self.type2test(Counter(self.word) - Counter(self.otherword))
        self.assertEqual(st, oracle)

    def test_isub(self):
        s2 = self.type2test(self.otherword)
        self.s -= s2
        oracle = self.type2test(Counter(self.word) - Counter(self.otherword))
        self.assertEqual(self.s, oracle)

    def test_lesseq(self):
        s2 = self.type2test('simsalabim')
        self.assertTrue(self.s <= s2)
        s2 = self.type2test('simsalabimsamsim')
        self.assertTrue(self.s <= s2)
        s2 = self.type2test('simsalabum')
        self.assertFalse(s2 <= self.s)

    def test_lessthan(self):
        s2 = self.type2test('simsalabim')
        self.assertFalse(self.s < s2)
        s2 = self.type2test('simsalabimsamsim')
        self.assertTrue(self.s < s2)
        s2 = self.type2test('simsalabum')
        self.assertFalse(s2 < self.s)

    def test_greatereq(self):
        s2 = self.type2test('simsalabim')
        self.assertTrue(s2 >= self.s)
        s2 = self.type2test('simsalabimsamsim')
        self.assertTrue(s2 >= self.s)
        s2 = self.type2test('simsalabum')
        self.assertFalse(self.s >= s2)

    def test_greaterthan(self):
        s2 = self.type2test('simsalabim')
        self.assertFalse(s2 > self.s)
        s2 = self.type2test('simsalabimsamsim')
        self.assertTrue(s2 > self.s)
        s2 = self.type2test('simsalabum')
        self.assertFalse(self.s > s2)

    def test_and(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k in dow:
                k_ = dow[k]
                and_val[k] = v if v <= k_ else k_
        i = self.type2test(and_val)
        self.assertEqual(self.s & self.type2test(self.otherword), i)

    def test_iand(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k in dow:
                k_ = dow[k]
                and_val[k] = v if v <= k_ else k_
        i = self.type2test(and_val)
        self.s &= self.type2test(self.otherword)
        self.assertEqual(self.s, i)

    def test_or(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k in dow:
                k_ = dow[k]
                and_val[k] = v if v >= k_ else k_
            else:
                and_val[k] = v
        for k, v in dow.items():
            if k not in and_val:
                and_val[k] = v
        i = self.type2test(and_val)
        self.assertEqual(self.s | self.type2test(self.otherword), i)

    def test_ior(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k in dow:
                k_ = dow[k]
                and_val[k] = v if v >= k_ else k_
            else:
                and_val[k] = v
        for k, v in dow.items():
            if k not in and_val:
                and_val[k] = v
        i = self.type2test(and_val)
        self.s |= self.type2test(self.otherword)
        self.assertEqual(self.s, i)

    def test_xor(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k not in dow:
                and_val[k] = v
        for k, v in dow.items():
            if k not in dw:
                and_val[k] = v
        i = self.type2test(and_val)
        self.assertEqual(self.s ^ self.type2test(self.otherword), i)

    def test_ixor(self):
        c = list(self.word)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dw = dict(list(x))
        c = list(self.otherword)
        x = zip(c, map(list.count, itertools.repeat(c), c))
        dow = dict(list(x))
        and_val = dict()
        for k, v in dw.items():
            if k not in dow:
                and_val[k] = v
        for k, v in dow.items():
            if k not in dw:
                and_val[k] = v
        i = self.type2test(and_val)
        self.s ^= self.type2test(self.otherword)
        self.assertEqual(self.s, i)

    def test_addmany(self):
        self.s.add('m', 3)
        self.assertEqual(self.s['m'], 5)

    def test_discardmany(self):
        self.s.discard('a', 2)
        self.assertEqual(self.s['a'], 0)
        self.assertNotIn('a', self.s)
        s = self.type2test(self.otherword)
        s.discard('a', 2)
        self.assertEqual(s['a'], 2)

    def test_setOfFrozensets(self):
        t = list(map(frozenset, ['abcdef', 'bbcd', 'bdcb', 'fed', 'fedcba']))
        s = self.type2test(t)
        self.assertEqual(len(s), 5)
        fs = t[0]
        self.assertEqual(s[fs], 2)

    def test_tally(self):
        pass


# -*- coding: utf-8 -*-
from .__version import __version__
from .mochila import Sequence, Set, OrderedSet, Bag, MultiSet
import logging

__author__ = 'Horacio Hoyos Rodriguez'
__copyright__ = 'Copyright , Kinori Technologies'

try:
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())


"""
Tests common to Sequences
"""
import sys

import pytest

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class SequenceTestBase:
    # The type to be tested
    type2test = None

    def test_getitem(self):
        u = self.type2test([0, 1, 2, 3, 4])
        for i in range(len(u)):
            self.assertEqual(u[i], i)
            self.assertEqual(u[int(i)], i)
        for i in range(-len(u), -1):
            self.assertEqual(u[i], len(u) + i)
            self.assertEqual(u[int(i)], len(u) + i)
        self.assertRaises(IndexError, u.__getitem__, -len(u) - 1)
        self.assertRaises(IndexError, u.__getitem__, len(u))
        self.assertRaises(ValueError, u.__getitem__, slice(0, 10, 0))

        u = self.type2test()
        self.assertRaises(IndexError, u.__getitem__, 0)
        self.assertRaises(IndexError, u.__getitem__, -1)

        self.assertRaises(TypeError, u.__getitem__)

        a = self.type2test([10, 11])
        self.assertEqual(a[0], 10)
        self.assertEqual(a[1], 11)
        self.assertEqual(a[-2], 10)
        self.assertEqual(a[-1], 11)
        self.assertRaises(IndexError, a.__getitem__, -3)
        self.assertRaises(IndexError, a.__getitem__, 3)

    def test_getslice(self):
        l = [0, 1, 2, 3, 4]
        u = self.type2test(l)
        self.assertEqual(u[0:0], self.type2test([]))
        self.assertEqual(u[1:2], self.type2test([1]))
        self.assertEqual(u[-2:-1], self.type2test([3]))
        self.assertEqual(u[-1000:1000], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[1000:-1000], self.type2test([]))
        self.assertEqual(u[:], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[1:None], self.type2test([1, 2, 3, 4]))
        self.assertEqual(u[None:3], self.type2test([0, 1, 2]))

        # Extended slices
        self.assertEqual(u[::], self.type2test(u))
        self.assertEqual(u[::2], self.type2test([0, 2, 4]))
        self.assertEqual(u[1::2], self.type2test([1, 3]))
        self.assertEqual(u[::-1], self.type2test([4, 3, 2, 1, 0]))
        self.assertEqual(u[::-2], self.type2test([4, 2, 0]))
        self.assertEqual(u[3::-2], self.type2test([3, 1]))
        self.assertEqual(u[3:3:-2], self.type2test([]))
        self.assertEqual(u[3:2:-2], self.type2test([3]))
        self.assertEqual(u[3:1:-2], self.type2test([3]))
        self.assertEqual(u[3:0:-2], self.type2test([3, 1]))
        self.assertEqual(u[::-100], self.type2test([4]))
        self.assertEqual(u[100:-100:], self.type2test([]))
        self.assertEqual(u[-100:100:], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[100:-100:-1], u[::-1])
        self.assertEqual(u[-100:100:-1], self.type2test([]))
        self.assertEqual(u[-100:100:2], self.type2test([0, 2, 4]))

        # Test extreme cases with long ints
        a = self.type2test([0, 1, 2, 3, 4])
        self.assertEqual(a[-pow(2, 128): 3], self.type2test([0, 1, 2]))
        self.assertEqual(a[3: pow(2, 145)], self.type2test([3, 4]))

        # Slice [:] is a copy
        u = self.type2test(l)
        a = u[:]
        a.append(5)
        self.assertFalse(5 in u)

    def test_contains(self):
        l = [1, 2, 3]
        u = self.type2test(l)
        for i in l:
            self.assertIn(i, u)
        for i in min(l) - 1, max(l) + 1:
            self.assertNotIn(i, u)

        self.assertRaises(TypeError, u.__contains__)

    def test_contains_fake(self):
        class AllEq:
            # Sequences must use rich comparison against each item
            # (unless "is" is true, or an earlier item answered)
            # So instances of AllEq must be found in all non-empty sequences.
            def __eq__(self, other):
                return True

            __hash__ = None  # Can't meet hash invariant requirements

        self.assertNotIn(AllEq(), self.type2test([]))
        self.assertIn(AllEq(), self.type2test([1]))

    def test_contains_order(self):
        # Sequences must test in-order.  If a rich comparison has side
        # effects, these will be visible to tests against later members.
        # In this test, the "side effect" is a short-circuiting raise.
        class DoNotTestEq(Exception):
            pass

        class StopCompares:
            def __eq__(self, other):
                raise DoNotTestEq

            def __hash__(self):
                return 1

        checkfirst = self.type2test([1, StopCompares()])
        self.assertIn(1, checkfirst)
        checklast = self.type2test([StopCompares(), 1])
        self.assertRaises(DoNotTestEq, checklast.__contains__, 1)

    def test_len(self):
        self.assertEqual(len(self.type2test()), 0)
        self.assertEqual(len(self.type2test([])), 0)
        self.assertEqual(len(self.type2test([0])), 1)
        self.assertEqual(len(self.type2test([0, 1, 2])), 3)

    def test_addmul(self):
        u1 = self.type2test([0])
        u2 = self.type2test([0, 1])
        with pytest.raises(TypeError):
            self.assertEqual(u1, u1 + self.type2test())
        with pytest.raises(TypeError):
            self.assertEqual(self.type2test(), u2 * 0)

    def test_iadd(self):
        u = self.type2test([0, 1])
        u += self.type2test()
        self.assertEqual(u, self.type2test([0, 1]))
        u += self.type2test([2, 3])
        self.assertEqual(u, self.type2test([0, 1, 2, 3]))
        u += self.type2test([4, 5])
        self.assertEqual(u, self.type2test([0, 1, 2, 3, 4, 5]))

        u = self.type2test("spam")
        u += self.type2test("eggs")
        self.assertEqual(u, self.type2test("spameggs"))

    def test_subscript(self):
        a = self.type2test([10, 11])
        self.assertEqual(a.__getitem__(0), 10)
        self.assertEqual(a.__getitem__(1), 11)
        self.assertEqual(a.__getitem__(-2), 10)
        self.assertEqual(a.__getitem__(-1), 11)
        self.assertRaises(IndexError, a.__getitem__, -3)
        self.assertRaises(IndexError, a.__getitem__, 3)
        self.assertEqual(a.__getitem__(slice(0, 1)), self.type2test([10]))
        self.assertEqual(a.__getitem__(slice(1, 2)), self.type2test( [11]))
        self.assertEqual(a.__getitem__(slice(0, 2)), self.type2test([10, 11]))
        self.assertEqual(a.__getitem__(slice(0, 3)), self.type2test([10, 11]))
        self.assertEqual(a.__getitem__(slice(3, 5)), self.type2test([]))
        self.assertRaises(ValueError, a.__getitem__, slice(0, 10, 0))
        self.assertRaises(TypeError, a.__getitem__, 'x')

    def test_count(self):
        a = self.type2test([0, 1, 2]*3)
        self.assertEqual(a.count(0), 3)
        self.assertEqual(a.count(1), 3)
        self.assertEqual(a.count(3), 0)

        self.assertRaises(TypeError, a.count)

        class BadExc(Exception):
            pass

        class BadCmp:
            def __eq__(self, other):
                if other == 2:
                    raise BadExc()
                return False

        self.assertRaises(BadExc, a.count, BadCmp())

    def test_index(self):
        u = self.type2test([0, 1])
        self.assertEqual(u.index(0), 0)
        self.assertEqual(u.index(1), 1)
        self.assertRaises(ValueError, u.index, 2)

        u = self.type2test([-2, -1, 0, 0, 1, 2])
        self.assertEqual(u.count(0), 2)
        self.assertEqual(u.index(0), 2)
        self.assertEqual(u.index(0, 2), 2)
        self.assertEqual(u.index(-2, -10), 0)
        self.assertEqual(u.index(0, 3), 3)
        self.assertEqual(u.index(0, 3, 4), 3)
        self.assertRaises(ValueError, u.index, 2, 0, -10)

        self.assertRaises(TypeError, u.index)

        class BadExc(Exception):
            pass

        class BadCmp:
            def __eq__(self, other):
                if other == 2:
                    raise BadExc()
                return False

        a = self.type2test([0, 1, 2, 3])
        self.assertRaises(BadExc, a.index, BadCmp())

        a = self.type2test([-2, -1, 0, 0, 1, 2])
        self.assertEqual(a.index(0), 2)
        self.assertEqual(a.index(0, 2), 2)
        self.assertEqual(a.index(0, -4), 2)
        self.assertEqual(a.index(-2, -10), 0)
        self.assertEqual(a.index(0, 3), 3)
        self.assertEqual(a.index(0, -3), 3)
        self.assertEqual(a.index(0, 3, 4), 3)
        self.assertEqual(a.index(0, -3, -2), 3)
        self.assertEqual(a.index(0, -4 * sys.maxsize, 4 * sys.maxsize), 2)
        self.assertRaises(ValueError, a.index, 0, 4 * sys.maxsize, -4 * sys.maxsize)
        self.assertRaises(ValueError, a.index, 2, 0, -10)

    def test_delitem(self):
        a = self.type2test([0, 1])
        del a[1]
        self.assertEqual(len(a), 1)
        del a[0]
        self.assertEqual(len(a), 0)

        a = self.type2test([0, 1])
        del a[-2]
        self.assertEqual(len(a), 1)
        self.assertTrue(1 in a)
        self.assertFalse(0 in a)
        del a[-1]
        self.assertEqual(len(a), 0)
        self.assertFalse(1 in a)

        a = self.type2test([0, 1])
        self.assertRaises(IndexError, a.__delitem__, -3)
        self.assertRaises(IndexError, a.__delitem__, 2)

        a = self.type2test([])
        self.assertRaises(IndexError, a.__delitem__, 0)

        self.assertRaises(TypeError, a.__delitem__)

    def test_reversed(self):
        a = self.type2test(range(20))
        r = reversed(a)
        self.assertEqual(list(r), list(range(19, -1, -1)))
        self.assertRaises(StopIteration, next, r)
        self.assertEqual(self.type2test(reversed(self.type2test())),
                         self.type2test())
        # Bug 3689: make sure list-reversed-iterator doesn't have __len__
        self.assertRaises(TypeError, len, reversed([1, 2, 3]))


import unittest
import weakref

import mochila
from test import test_set_base
from test import test_mochila

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class PassThru(Exception):
    pass


def check_pass_thru():
    raise PassThru
    yield 1


class SetTest(test_set_base.SetTestBase, test_mochila.MochilaTest, unittest.TestCase):

    type2test = mochila.Set
    basetype = mochila.Set

    def test_init(self):
        s = self.type2test()
        s.__init__(self.word)
        self.assertEqual(s._items, set(self.word))
        s.__init__(self.otherword)
        self.assertEqual(s._items, set(self.otherword))

        a = self.type2test([1, 2, 3])
        b = self.type2test(a)
        self.assertNotEqual(id(a), id(b))
        self.assertEqual(a, b)

        # FIXME need to test that args and kwargs can be passed
        self.assertRaises(TypeError, s.__init__, s, 2)
        self.assertRaises(TypeError, s.__init__, a=1)

    def test_constructor_identity(self):
        s = self.type2test(range(3))
        t = self.type2test(s)
        self.assertNotEqual(id(s), id(t))

    def test_hash(self):
        self.assertRaises(TypeError, hash, self.s)

    def test_clear(self):
        self.s.clear()
        self.assertEqual(self.s, self.type2test())
        self.assertEqual(len(self.s), 0)

    def test_copy(self):
        dup = self.s.copy()
        self.assertEqual(self.s, dup)
        self.assertNotEqual(id(self.s), id(dup))
        self.assertEqual(type(dup), self.basetype)

    def test_add(self):
        self.s.add('Q')
        self.assertIn('Q', self.s)
        dup = self.s.copy()
        self.s.add('Q')
        self.assertEqual(self.s, dup)
        self.assertRaises(TypeError, self.s.add, [])

    def test_remove(self):
        self.s.remove('a')
        self.assertNotIn('a', self.s)
        self.assertRaises(KeyError, self.s.remove, 'Q')
        self.assertRaises(TypeError, self.s.remove, [])

    def test_discard(self):
        self.s.discard('a')
        self.assertNotIn('a', self.s)
        self.s.discard('Q')
        self.assertRaises(TypeError, self.s.discard, [])

    def test_pop(self):
        for i in range(len(self.s)):
            elem = self.s.pop()
            self.assertNotIn(elem, self.s)
        self.assertRaises(KeyError, self.s.pop)

    def test_ior(self):
        self.s |= set(self.otherword)
        for c in (self.word + self.otherword):
            self.assertIn(c, self.s)

    def test_iand(self):
        self.s &= set(self.otherword)
        for c in (self.word + self.otherword):
            if c in self.otherword and c in self.word:
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_isub(self):
        self.s -= set(self.otherword)
        for c in (self.word + self.otherword):
            if c in self.word and c not in self.otherword:
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_ixor(self):
        self.s ^= set(self.otherword)
        for c in (self.word + self.otherword):
            if (c in self.word) ^ (c in self.otherword):
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_inplace_on_self(self):
        t = self.s.copy()
        t |= t
        self.assertEqual(t, self.s)
        t &= t
        self.assertEqual(t, self.s)
        t -= t
        self.assertEqual(t, self.type2test())
        t = self.s.copy()
        t ^= t
        self.assertEqual(t, self.type2test())

    def test_weakref(self):
        s = self.type2test('gallahad')
        p = weakref.proxy(s)
        self.assertEqual(str(p), str(s))
        s = None
        self.assertRaises(ReferenceError, str, p)

    def test_rich_compare(self):
        class TestRichSetCompare:
            def __gt__(self, some_set):
                self.gt_called = True
                return False

            def __lt__(self, some_set):
                self.lt_called = True
                return False

            def __ge__(self, some_set):
                self.ge_called = True
                return False

            def __le__(self, some_set):
                self.le_called = True
                return False

        # This first tries the builtin rich set comparison, which doesn't know
        # how to handle the custom object. Upon returning NotImplemented, the
        # corresponding comparison on the right object is invoked.
        myset = {1, 2, 3}

        myobj = TestRichSetCompare()
        myset < myobj
        self.assertTrue(myobj.gt_called)

        myobj = TestRichSetCompare()
        myset > myobj
        self.assertTrue(myobj.lt_called)

        myobj = TestRichSetCompare()
        myset <= myobj
        self.assertTrue(myobj.ge_called)

        myobj = TestRichSetCompare()
        myset >= myobj
        self.assertTrue(myobj.le_called)

    @unittest.skipUnless(hasattr(set, "test_c_api"),
                         'C API test only available in a debug build')
    def test_c_api(self):
        self.assertEqual(set().test_c_api(), True)

    def test_equals(self):
        self.assertEqual(self.type2test([]), self.type2test())
        s0_3 = self.type2test([0, 1, 2, 3])
        s0_3_bis = self.type2test(list(s0_3))
        self.assertEqual(s0_3, s0_3_bis)
        self.assertTrue(s0_3 is not s0_3_bis)
        self.assertEqual(s0_3, s0_3)
        self.assertFalse(s0_3 == '0123')
        self.assertTrue(self.type2test([]) != '0123')
        self.assertFalse(s0_3 != s0_3)
        s0_4 = self.type2test([0, 1, 2, 3, 4])
        self.assertNotEqual(s0_3, s0_4)
        s1_4 = self.type2test([1, 2, 3, 4])
        self.assertNotEqual(s0_3, s1_4)



import unittest
import pytest

import mochila
from test import test_mochila

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class PassThru(Exception):
    pass


def check_pass_thru():
    raise PassThru
    yield 1


class BagTest(test_mochila.MochilaTest, unittest.TestCase):

    type2test = mochila.Bag
    basetype = mochila.Bag

    def setUp(self):
        self.word = word = 'simsalabim'

    def test_init(self):
        # Iterable arg is optional
        self.assertEqual(self.type2test([]), self.type2test())

        a = self.type2test([1, 2, 3])
        b = self.type2test(a)
        self.assertNotEqual(id(a), id(b))
        self.assertEqual(a, b)

        # Init clears previous values
        a = self.type2test([1, 2, 3])
        a.__init__()
        self.assertEqual(a, self.type2test([]))

        # Init overwrites previous values
        a = self.type2test([1, 2, 3])
        a.__init__([4, 5, 6])
        self.assertEqual(a, self.type2test([4, 5, 6]))

        l = self.type2test()
        self.assertRaises(TypeError, l.__init__, (), 2)
        self.assertRaises(TypeError, l.__init__, (), a=12)

    def test_getitem(self):
        u = self.type2test([0, 1, 2, 3, 4])
        #self.assertRaises(IndexError, u.__getitem__, -len(u) - 1)

    def test_contains(self):
        l = [1, 2, 3]
        u = self.type2test(l)
        for i in l:
            self.assertIn(i, u)
        for i in min(l) - 1, max(l) + 1:
            self.assertNotIn(i, u)

        self.assertRaises(TypeError, u.__contains__)

    def test_contains_fake(self):
        class AllEq:
            # Sequences must use rich comparison against each item
            # (unless "is" is true, or an earlier item answered)
            # So instances of AllEq must be found in all non-empty sequences.
            def __eq__(self, other):
                return True

            __hash__ = None  # Can't meet hash invariant requirements

        self.assertNotIn(AllEq(), self.type2test([]))
        self.assertIn(AllEq(), self.type2test([1]))

    def test_len(self):
        self.assertEqual(len(self.type2test()), 0)
        self.assertEqual(len(self.type2test([])), 0)
        self.assertEqual(len(self.type2test([0])), 1)
        self.assertEqual(len(self.type2test([0, 1, 2])), 3)
        self.assertEqual(len(self.type2test([0, 1, 2, 2])), 4)

    def test_addmul(self):
        u1 = self.type2test([0])
        u2 = self.type2test([0, 1])
        with pytest.raises(TypeError):
            self.assertEqual(u1, u1 + self.type2test())
        with pytest.raises(TypeError):
            self.assertEqual(self.type2test(), u2 * 0)

    def test_iadd(self):
        u = self.type2test([0, 1])
        u += self.type2test()
        self.assertEqual(u, self.type2test([0, 1]))
        u += self.type2test([2, 3])
        self.assertEqual(u, self.type2test([0, 1, 2, 3]))
        u += self.type2test([4, 5])
        self.assertEqual(u, self.type2test([0, 1, 2, 3, 4, 5]))

        u = self.type2test("spam")
        u += self.type2test("eggs")
        self.assertEqual(u, self.type2test("spameggs"))

    def test_count(self):
        a = self.type2test([0, 1, 2]*3)
        self.assertEqual(a.count(0), 3)
        self.assertEqual(a.count(1), 3)
        self.assertEqual(a.count(3), 0)

        self.assertRaises(TypeError, a.count)

        class BadExc(Exception):
            pass

        class BadCmp:
            def __eq__(self, other):
                if other == 2:
                    raise BadExc()
                return False

        self.assertRaises(BadExc, a.count, BadCmp())

    def test_pop(self):
        a = self.type2test([-1, 0, 1])
        e = a.pop()
        self.assertEqual(len(a), 2)
        self.assertTrue(e not in a)
        e = a.pop()
        self.assertEqual(len(a), 1)
        self.assertTrue(e not in a)
        e = a.pop()
        self.assertEqual(a, self.type2test())
        self.assertEqual(len(a), 0)
        self.assertTrue(e not in a)
        self.assertRaises(KeyError, a.pop)

    def test_remove(self):
        a = self.type2test([0, 0, 1])
        a.remove(1)
        self.assertEqual(a, self.type2test([0, 0]))
        a.remove(0)
        self.assertEqual(a, self.type2test([0]))
        a.remove(0)
        self.assertEqual(a, self.type2test([]))
        self.assertRaises(ValueError, a.remove, 0)
        self.assertRaises(TypeError, a.remove)

        class BadExc(Exception):
            pass

        class BadCmp:
            def __eq__(self, other):
                if other == 2:
                    raise BadExc()
                return False

        a = self.type2test([0, 1, 2, 3])
        self.assertRaises(BadExc, a.remove, BadCmp())

        class BadCmp2:
            def __eq__(self, other):
                raise BadExc()

        d = self.type2test('abcdefghcij')
        d.remove('c')
        self.assertEqual(d, self.type2test('abdefghcij'))
        d.remove('c')
        self.assertEqual(d, self.type2test('abdefghij'))
        self.assertRaises(ValueError, d.remove, 'c')
        self.assertEqual(d, self.type2test('abdefghij'))

        # Handle comparison errors
        d = self.type2test(['a', 'b', BadCmp2(), 'c'])
        e = self.type2test(d)
        self.assertRaises(BadExc, d.remove, 'c')

    def test_equals(self):
        self.assertEqual(self.type2test([]), self.type2test())
        b0_3 = self.type2test([0, 1, 2, 3])
        b0_3_bis = self.type2test(list(b0_3))
        self.assertEqual(b0_3, b0_3_bis)
        self.assertTrue(b0_3 is not b0_3_bis)
        self.assertEqual(b0_3, b0_3)
        self.assertFalse(b0_3 == '0123')
        self.assertTrue(self.type2test([]) != '0123')
        self.assertFalse(b0_3 != b0_3)
        b0_4 = self.type2test([0, 1, 2, 3, 4])
        self.assertNotEqual(b0_3, b0_4)
        b1_4 = self.type2test([1, 2, 3, 4])
        self.assertNotEqual(b0_3, b1_4)

    def test_copy(self):
        u = self.type2test([1, 2, 3])
        v = u.copy()
        self.assertEqual(v, self.type2test([1, 2, 3]))

        u = self.type2test([])
        v = u.copy()
        self.assertEqual(v, self.type2test([]))

        # test that it's indeed a copy and not a reference
        u = self.type2test(['a', 'b'])
        v = u.copy()
        v.add('i')
        self.assertTrue('a' in v)
        self.assertTrue('b' in v)
        self.assertTrue('i' in v)
        self.assertFalse('i' in u)

        # test that it's a shallow, not a deep copy
        u = self.type2test([1, 2, [3, 4], 5])
        v = u.copy()
        self.assertEqual(u, v)
        for x in u:
            try:
                if len(x):
                    ux = x
                    break
            except TypeError:
                pass
        for x in v:
            try:
                if len(x):
                    uv = x
                    break
            except TypeError:
                pass
        self.assertIs(ux, uv)
        self.assertRaises(TypeError, u.copy, None)


import io
import os

from setuptools import setup

import mochila

here = os.path.abspath(os.path.dirname(__file__))


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

long_description = read('README.rst', 'CHANGELOG.rst')

setup(
    name='mochila',
    version=mochila.__version__,
    packages=['mochila'],
    include_package_data=True,
    package_data={'mochila': ['CONTRIB.rst', 'CHANGELOG.rst']},
    # Metadata
    author='Horacio Hoyos Rodriguez',
    author_email='arcanefoam@gmail.com',
    description='Provides collections and a powerful API to process data in those collections in a declarative way.',
    url='http://bitbucket.org/arcanefoam/mochila',
    long_description=long_description,
    platforms='any',
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 5 - Production/Stable',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
)



# -*- coding: utf-8 -*-
from abc import ABCMeta
import collections
import itertools
import random
import warnings

"""
Reflective API
==============

The main goal of the Mochila implementation is to support reflective APIs. In reflective APIs (for 
example the OMG's MOF) the collection is used to represent an object's attribute and the object needs to be aware
of changes in the collection. This is not possible with standard collections. For example::

    >>> class Project():
    ...     def __init__():
    ...         self.members = []
    
    >>> p = Project()
    >>> p.members.append("Albert")
    >>> p.members.append("Julian")
    >>> p.members.remove("Albert")

In this case, the Project is not aware of changes done to the collection. You could add an "addMember" or other methods
in order to have control over the contents, but it is not pythonic. Further, in reflective API's it also desirable that 
objects know when they have been added/removed to/from a collection. Thus, the collection should be able to use the
object's notification mechanism in order to inform these changes.

Currently the reflective API is the responsibility of the class/script using mochilas (e.g. using decorators to keep
track of methods that modify the collection). To facilitate this approach, the Mochila's API objective is to reduce the
number of methods that modify the Mochila so that the module that uses the Mochila has a small, clear set of methods 
that must be overridden/decorated. Thus, some of the methods may appear sub-optimal, i.e. reusing existing
methods rather than accessing the underlying native collections instead. 

In the future, the reflective API might be incorporated directly into the Mochila API.

For each of the Mochilas, the set of methods that perform modifications are:
Sequence
    __set_item__, __del_item__, insert
Set
    add, discard
OrderedSet
    __set_item__, __del_item__, insert, discard
Bag
    add, remove
MultiSet
    Is not part of the reflective collections

"""

__author__ = 'Horacio Hoyos Rodriguez'
__copyright__ = 'Copyright , Kinori Technologies'

SLICE_ALL = slice(None)

warnings.simplefilter('default', UserWarning)


def _is_iterable(obj):
    """
    Are we being asked to look up a list of things, instead of a single thing?
    We check for the `__iter__` attribute so that this can cover types that
    don't have to be known by this module, such as NumPy arrays.
    Strings, however, should be considered as atomic values to look up, not
    iterables. The same goes for tuples, since they are immutable and therefore
    valid entries.
    We don't need to check for the Python 2 `unicode` type, because it doesn't
    have an `__iter__` attribute anyway.
    """
    return hasattr(obj, '__iter__') and not isinstance(obj, str) and not isinstance(obj, tuple)


def _closure_decorate(func):
    def closure_wrapper(x):
        if _is_iterable(x):
            cl = map(func, x)
            return list(cl)
        else:
            return func(x)
    return closure_wrapper


def _closure(iterable, f):
    cl = map(f, iterable)
    result = Bag((Bag(x) for x in cl if x is not None))
    _closure_recursion(f, result)
    return result


def _closure_recursion(f, result):
    for sr in result:
        if sr:
            sr_cl = _closure(sr, f)
            sr.add_all(sr_cl)


class MochilaError(TypeError):
    """Thrown to indicate errors in the use of a Mochila operation."""
    pass


class Mochila:
    """
    Define common operations. This class is not meant to be instantiated.
    """

    # Declarative operations

    def aggregate(self, key, value):
        def key_value_gen(x):
            yield key(x)
            yield value(x)

        return dict(map(key_value_gen, self))

    def closure(self, f):
        f = _closure_decorate(f)
        cl = map(f, self)
        result = Bag([Bag([x]) if x is not None else Bag([]) for x in cl])
        _closure_recursion(f, result)
        return result

    def collect(self, f):
        return Bag(map(f, self))

    def exists(self, f):
        return any(map(f, self))

    def for_all(self, f):
        return all(map(f, self))

    def one(self, f):
        r = [x for x in self if f(x)]
        return len(r) == 1

    def reject(self, f):
        r = (x for x in self if not f(x))
        return self.__class__(r)

    def select(self, f):
        #r = (x for x in self if f(x))
        r = filter(f, self)
        return self.__class__(r)

    def select_one(self, f, default=None):
        for x in self:
            if f(x):
                return x
        return default

    def sort_by(self, f, reverse=None, inplace=False):
        if reverse is None:
            reverse = 0
        if inplace:
            if isinstance(self, Set) or isinstance(self, Bag) or isinstance(self, MultiSet):
                warnings.warn("You can not sort an unordered Mochila in-place. The operation will be ignored",
                              UserWarning)
                return
            self.sort(f, reverse)
        else:
            return Sequence(sorted(self, key=f, reverse=reverse))

    # Core operations

    def _flatten(self):
        for x in self:
            if isinstance(x, Mochila):
                yield from x.flatten()
            elif not _is_iterable(x):
                yield x
            else:
                for x2 in x:
                    yield x2

    def add_all(self, iterable):
        if self is iterable:
            for x in iterable[:]:
                self.add(x)
        else:
            for v in iterable:
                self.add(v)

    def discard_all(self, iterable):
        for x in iterable:
            self.discard(x)

    # FIXME can specific collections provide better implementations? E.g. an empty set intersection
    def excludes_all(self, iterable):
        for x in iterable:
            if x in self:
                break
        else:
            return True
        return False

    def excluding(self, x):
        copy = self.copy()
        copy.remove(x)
        return copy

    def excluding_all(self, x):
        copy = self.copy()
        copy.remove(x)
        return copy

    def flatten(self):
        return Bag(self._flatten())

    def includes_all(self, iterable):
        for x in iterable:
            if x not in self:
                break
        else:
            return True
        return False

    def including(self, x):
        copy = self.copy()
        copy.add(x)
        return copy

    def including_all(self, iterable):
        copy = self.copy()
        copy.add_all(iterable)
        return copy

    def remove_all(self, iterable):
        for x in iterable:
            self.remove(x)

    def as_bag(self):
        return Bag(self)

    def as_multiset(self):
        return MultiSet(self)

    def as_set(self):
        return Set(self)

    def as_sequence(self, key=None, reverse=None):
        l = sorted(self, key, reverse)
        return Sequence(l)

    def as_orderedset(self, key=None, reverse=None):
        if key is not None and reverse is not None:
            l = sorted(self, key=key, reverse=reverse)
        elif key is not None:
            l = sorted(self, key=key)
        elif reverse is not None:
            l = sorted(self, reverse=reverse)
        else:
            l = sorted(self)
        return OrderedSet(l)


class Sequence(collections.MutableSequence, Mochila):
    """
    A Sequence is an enumerated collection of items. The order in which the items appear in the collection matters
    and the same item can appear multiple times.

    A sequence behaves as the Python built-in *list* type. The main difference is that item access by index additionally
    supports an iterable as the index. In this case the result will be a Sequence of the items corresponding to those
    indices.

    >>> m = Sequence(['a','b','c'])                           # a new Sequence
    >>> m[1]
    'b'
    >>> m[[0,2]]
    ['a', 'c']

    """
    def __init__(self, iterable=()):
        """
        Sequences can be constructed using the type constructor `Sequence()` or `Sequence(iterable)`. The constructor
        builds a Sequence whose items are the same and in the same order as *iterable*‘s items. *iterable* may be
        either a sequence, a container that supports iteration, or an iterator object. For example, `Sequence('abc')`
        returns `['a', 'b', 'c']` and `Sequence((1, 2, 3))` returns `[1, 2, 3]`. If no argument is given, the
        constructor creates a new empty Sequence, [].

        Note that the default representation of a Sequence is the same as for a *list*.

        :param iterable: An iterable used to instantiate the Sequence
        """
        self._items = list(iterable)

    def __getitem__(self, key):
        """
        Get the item at a given key, called to implement the evaluation of `self[key]`.
        If `key` is a slice, you will get back that slice of items. If it's the slice [:], a copy of the Sequence is
        returned.
        If `key` is an iterable, you'll get the Sequence of items corresponding to those indices (this is similar to
        NumPy's "fancy indexing").

        :param key: Index or slice or list of the item(s) to get
        """
        if key == SLICE_ALL:
            return Sequence(self._items)
        elif hasattr(key, '__index__') or isinstance(key, slice):
            result = self._items[key]
            if isinstance(key, slice):
                if isinstance(result, list):
                    return Sequence(result)
            return result
        elif _is_iterable(key):
            return Sequence([self._items[i] for i in key])
        else:
            raise TypeError('Don\'t know how to key an Sequence by {!r}'.format(key))

    def __setitem__(self, index, value):
        return self._items.__setitem__(index, value)

    def __delitem__(self, index):
        return self._items.__delitem__(index)

    def __len__(self):
        return self._items.__len__()

    def __iter__(self):
        return self._items.__iter__()

    def __eq__(self, other):
        if self is other:
            return True
        if isinstance(other, self.__class__):
            return len(self) == len(other) and self._items == other._items
        return NotImplemented

    def __ne__(self, other):
        if self is other:
            return False
        if isinstance(other, self.__class__):
            if len(self) != len(other):
                return True
            else:
                return self._items != other._items
        return NotImplemented

    def __str__(self):
        return str(self._items)

    def __repr__(self):
        return repr(self._items)

    def add(self, item):
        return self.append(item)

    def copy(self):
        return Sequence(self._items)

    def extend(self, iterable):
        if self is iterable:
            for i in range(len(iterable)):
                self.append(iterable[i])
        else:
            for v in iterable:
                self.append(v)

    def index(self, value, start=0, stop=None):
        return super().index(value, start, stop)

    def insert(self, index, value):
        self._items.insert(index, value)

    def sort(self, key=None, reverse=False):
        self._items.sort(key=key, reverse=reverse)

    def discard(self, value, n_copies=1):
        for n in range(n_copies):
            try:
                i = self.index(value)
                del self[i]
            except ValueError:
                pass


class Set(collections.MutableSet, Mochila):
    """
    A Set is a collection of items. The order in which the items appear in the collection is irrelevant and the same
    item can appear only once in the collection. Items in the collection must be `hashable <https://docs.python.org/3.6/glossary.html#term-hashable>`_.

    A Set behaves as the Python built-in *set* type. The main difference is that the non-operator versions of
    the update(), intersection_update(), difference_update(), and symmetric_difference_update() are not supported.
    """

    def __init__(self, iterable=()):
        """
        Sets can be constructed using the type constructor `Set()` or `Set(iterable)`. The constructor builds a Set
        whose items are taken from *iterable*.  To represent sets of sets, the inner sets must be `frozenset <https://docs.python.org/3/library/stdtypes.html?highlight=list#frozenset>`_
        objects. If *iterable* is not specified, a new empty Set is returned.

        :param iterable: An iterable from which items are taken to build the Set.
        """
        self._items = set(iterable)

    def add(self, value):
        self._items.add(value)

    def discard(self, value):
        if value not in self:
            return
        self._items.discard(value)

    def __len__(self):
        return self._items.__len__()

    def __iter__(self):
        return self._items.__iter__()

    def __contains__(self, value):
        return self._items.__contains__(value)

    def __eq__(self, other):
        if self is other:
            return True
        return super().__eq__(other)

    def __ne__(self, other):
        if self is other:
            return False
        return not self == other

    def __str__(self):
        return str(self._items)

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self._items)

    def copy(self):
        cp = Set(self._items)
        return cp

    def clear(self):
        return self._items.clear()


class OrderedSet(collections.MutableSet, collections.MutableSequence, Mochila):
    """
    An OrderedSet is an enumerated collection of items. The order in which the items appear in the collection matters
    and the same item can appear only once in the collection. Items in the collection must be `hashable <https://docs.python.org/3.6/glossary.html#term-hashable>`_.

    An OrderedSet behaves as the Python built-in *set* type. The main difference is that the non-operator versions of
    the update(), intersection_update(), difference_update(), and symmetric_difference_update() are not supported.

    Additionally, as for Sequences, item access by index additionally supports an iterable as the index. In this case
    the result will be a Sequence of the items corresponding to those indices.

    >>> m = OrderedSet('madagascar')
    >>> print(m)
    ['m', 'a', 'd', 'g', 's', 'c', 'r']
    >>> print(m[[0,2,5]])
    ['m', 'd', 'c']

    """
    #The OrderSet has O(1) search, O(n) insert, O(n) delete
    def __init__(self, iterable=None):
        """
        OrderedSets can be constructed using the type constructor `OrderedSet()` or `OrderedSet(iterable)`. The
        constructor builds an OrderedSet whose items are taken from *iterable*. To represent sets of sets, the inner
        sets must be `frozenset <https://docs.python.org/3/library/stdtypes.html?highlight=list#frozenset>`_ objects.
        If *iterable* is not specified, a new empty OrderedSet is returned.

        :param iterable: An iterable from which items are taken to build the OrderedSet.
        """
        self._items = []
        self._map = {}
        if iterable is not None:
            self |= iterable

    def __len__(self):
        return len(self._items)

    def __getitem__(self, key):
        """
        Get the item at a given key, called to implement the evaluation of `self[key]`.
        If `key` is a slice, you will get back that slice of items. If it's the slice [:], a copy of the object is
        returned. If `key` is an iterable, you'll get the OrderedSet of items corresponding to those indices (this is
        similar to NumPy's  "fancy indexing".).

        :param key: Index or slice or list of the item(s) to get
        :return: a single element (for index) or an OrderedSet of elements (for slice and list)
        """
        if key == SLICE_ALL:
            return self._from_iterable(self._items)
        elif hasattr(key, '__index__') or isinstance(key, slice):
            result = self._items[key]
            if isinstance(result, list):
                return self._from_iterable(result)
            else:
                return result
        elif _is_iterable(key):
            return self._from_iterable([self._items[i] for i in key])
        else:
            raise TypeError("Don't know how to key an OrderedSet by %r" % key)

    def __setitem__(self, index, value):
        """
        Set the item at a given index, called to implement assignment to `self[key]`. If the value is already in the
        set, it is unmodified.

        :param index: the index
        :param value: the new value
        :return: the old value at the index
        """
        if value not in self._map:
            old_value = self._items[index]
            self._items.__setitem__(index, value)
            self._map.pop(old_value)
            self._map[value] = index
        return

    def __delitem__(self, index):
        """
        Delete the element at the given index, called to implement deletion of `self[key]`.

        .. note::
            This method is O(n) because the implementation keeps the items and the indexes in separate structures.

        :param index: the index
        """
        value = self._items.pop(index)
        self._map.pop(value)
        for v in self._items[index:]:
            self._map[v] += 1
        return

    def __iter__(self):
        return iter(self._items)

    def __reversed__(self):
        return reversed(self._items)

    def __contains__(self, key):
        return key in self._map

    def __eq__(self, other):
        if self is other:
            return True
        if isinstance(other, self.__class__):
            return len(self) == len(other) and self.__le__(other)
        return NotImplemented

    def __ne__(self, other):
        if self is other:
            return False
        return not self == other

    def __str__(self):
        return str(self._items)

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self._items)

    def add(self, item):
        super().append(item)

    def discard(self, key):
        if key in self:
            i = self._map[key]
            self._items.pop(i)
            self._map.pop(key)
            for k, v in self._map.items():
                if v >= i:
                    self._map[k] = v - 1

    def insert(self, index, x):
       if x not in self:
            self._items.insert(index, x)
            for k, v in self._map.items():
                if v >= index:
                    self._map[k] = v + 1
            self._map[x] = index

    def index(self, x, start=0, stop=None):
        """
        Return the index in the list of the item whose value is x. It is an error if there is no such item.
        If x is an iterable that is not a string, it returns a list of indices.
        Since items in a set are unique, the start and stop parameters are always ignored.

        :param x:
        :param start:
        :param stop:
        :return:
        """
        if _is_iterable(x):
            return [self.index(subkey) for subkey in x]
        try:
            return self._map[x]
        except KeyError as ex:
            raise ValueError from ex

    def copy(self):
        return OrderedSet(self)

    def pop(self, index=None):
        """
        If index is none, it behaves like set.pop, O(n)
        Else, it behaves like list pop.

        :param index:
        :return:
        """
        if index is not None:
            elem = self[index]
            self.discard(elem)
        else:
            elem = super().pop()
        return elem

    def sort(self, key=None, reverse=False):
        self._items.sort(key=key, reverse=reverse)
        for i, x in enumerate(self._items):
            self._map[x] = i

    # Enable if we want to enable pickle
    # def __getstate__(self):
    #     if len(self) == 0:
    #         # The state can't be an empty list.
    #         # We need to return a truthy value, or else __setstate__ won't be run.
    #         #
    #         # This could have been done more gracefully by always putting the state
    #         # in a tuple, but this way is backwards- and forwards- compatible with
    #         # previous versions of OrderedSet.
    #         return (None,)
    #     else:
    #         return list(self)
    #
    # def __setstate__(self, state):
    #     if state == (None,):
    #         self.__init__([])
    #     else:
    #         self.__init__(state)


class Bag(collections.Iterable, Mochila):
    """
    A Bag is a collection of objects. The order in which the appear in the collection is irrelevant and the same
    object can appear multiple times.

    A Bag behaves as the Python built-in *list* type, without the index access capabilities. Hence, a bag is the most
    basic implementation of an iterable.
    """
    def __init__(self, iterable=()):
        """
        Bags can be constructed using the type constructor `Bag()` or `Bag(iterable)`. The constructor builds a Bag
        whose items are taken from *iterable*. If *iterable* is not specified, a new empty Bag is returned.

        :param iterable: An iterable from which items are taken to build the Bag.
        """
        self._items = list(iterable)

    def __len__(self):
        return len(self._items)

    def __iter__(self):
        order = random.sample(range(len(self._items)), k=len(self._items))
        for i in order:
            yield self._items[i]

    def __contains__(self, value):
        for v in self:
            if v is value or v == value:
                return True
        return False

    def __eq__(self, other):
        if self is other:
            return True
        if isinstance(other, self.__class__):
            if len(self) == len(other):
                unmatched = list(other)
                for element in self:
                    try:
                        unmatched.remove(element)
                    except ValueError:
                        return False
                return True
        return NotImplemented

    def __ne__(self, other):
        if self is other:
            return False
        if isinstance(other, self.__class__):
            if len(self) != len(other):
                return True
            else:
                unmatched = list(other)
                for element in self:
                    try:
                        unmatched.remove(element)
                    except ValueError:
                        return True
                return False
        return NotImplemented

    def __iadd__(self, values):
        self._items.extend(values)
        return self

    def __str__(self):
        return str(self._items)

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self._items)

    def add(self, value):
        """Add an element."""
        self._items.append(value)

    def count(self, value):
        """S.count(value) -> integer -- return number of occurrences of value"""
        return sum(1 for v in self if v == value)

    def clear(self):
        """S.clear() -> None -- remove all items from S"""
        try:
            while True:
                self.pop()
        except KeyError:
            pass

    def copy(self):
        """Create a copy of the Bag."""
        return Bag(self._items)

    def pop(self):
        """
        Pop a random item from the bag and return it. Raise KeyError if empty.
        :return:
        """
        #O(n) worst case remove first items, all need to be shifted"""
        it = iter(self)
        try:
            value = next(it)
        except StopIteration:
            raise KeyError
        self.remove(value)
        return value

    def remove(self, value):
        """
        Remove an occurrence of value. Raise ValueError if the value is not present.
        :param value:
        :return:
        """
        return self._items.remove(value)

    def discard(self, value):
        """Remove an element.  Do not raise an exception if absent."""
        try:
            self.remove(value)
        except ValueError:
            pass


class MultiSet(collections.MutableSet, Mochila):
    """
    A MultiSet is a collection of items. The order in which the items appear in the collection is irrelevant and the
    same item can appear multiple times. Items in the collection must be `hashable <https://docs.python.org/3.6/glossary.html#term-hashable>`_.
    A MultiSet is a generalization of the concept of a set that, unlike a set, allows multiple instances of the same
    element.

    """

    def __init__(*args, **kwds):
        """
        MultiSets can be constructed using the type constructor `MultiSet()` or from the count of elements from an
        iterable, or initialize the MultiSet from another mapping of elements to their counts (the counts must be
        numbers).

        >>> m = MultiSet()                           # a new, empty MultiSet
        >>> m = MultiSet('gallahad')                 # a new MultiSet from an iterable
        >>> m = MultiSet({'a': 4, 'b': 2})           # a new MultiSet from a mapping
        >>> m = MultiSet(a=4, b=2)                   # a new MultiSet from keyword args

        :param args:
        :param kwds:
        """
        if not args:
            raise TypeError("descriptor '__init__' of 'MultiSet' object needs an argument")
        self, *args = args
        self._items = collections.Counter(*args, **kwds)

    def __contains__(self, value):
        """
        O(n)
        :param value: 
        :return: 
        """
        return value in self._items

    def __len__(self):
        return sum(self._items.values())

    def __iter__(self):
        for i in self._items.elements():
            yield i

    def __getitem__(self, key):
        """
        Get the count for the given key, called to implement the evaluation of `self[key]`.
        If `key` is an iterable, you'll get the Sequence of counts corresponding to those keys (this is similar to
        NumPy's "fancy indexing").

        :param key: Index or list of the item(s) to get
        """
        if _is_iterable(key):
            if isinstance(key, frozenset):
                return self._items[key]
        return self._items[key]

    def __setitem__(self, index, value):
        """
        Set the count for a given key, called to implement the evaluation of `self[key] = value`.
        :param index:
        :param value:
        :return:
        """
        return self._items.__setitem__(index, value)

    def __delitem__(self, index):
        return self._items.__delitem__(index)

    def __missing__(self, key):
        'The count of elements not in the Counter is zero.'
        # Needed so that self[missing_item] does not raise KeyError
        return 0

    def __add__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return MultiSet(self._items.__add__(other._items))

    __radd__ = __add__

    def __iadd__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        self._items.__iadd__(other._items)
        return self

    def __sub__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return MultiSet(self._items.__sub__(other._items))

    __rsub__ = __sub__

    def __isub__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        self._items.__isub__(other._items)
        return self

    def __le__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        if len(self) > len(other):
            return False
        for elem in self:
            if elem not in other:
                return False
            if self[elem] > other[elem]:
                return False
        return True

    def __lt__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return len(self) < len(other) and self.__le__(other)

    def __ge__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        if len(self) < len(other):
            return False
        for elem in other:
            if elem not in self:
                return False
            if self[elem] < other[elem]:
                return False
        return True

    def __gt__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return len(self) > len(other) and self.__ge__(other)

    def __and__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return MultiSet(self._items & other._items)

    __rand__ = __and__

    def __or__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        return MultiSet(self._items | other._items)

    __ror__ = __or__

    def __iand__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        self._items &= other._items
        return self

    def __ior__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        self._items |= other._items
        return self

    def __xor__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        keep = self._items.keys() ^ other._items.keys()
        result = MultiSet({k: self._items[k] for k in keep if k in self._items})
        result._items.update({k: other._items[k] for k in keep if k in other._items})
        return result

    def __ixor__(self, other):
        if not isinstance(other, MultiSet):
            return NotImplemented
        keep = self._items.keys() ^ other._items.keys()
        self_pop = self._items.pop
        for k in other._items.keys():
            self_pop(k, None)
        self._items.update({k: other._items[k] for k in keep if k in other._items})
        return self

    # def __repr__(self):
    #     return 'MultiSet({})'.format(list(self.elements()))
    #
    # def __str__(self):
    #     format_str = '{}[{}]'.format
    #     return 'MultiSet({})'.format(', '.join(format_str(k, v) for k, v in self._items()))

    def add(self, value, n_copies=1):
        self._items[value] = self._items[value] + n_copies

    def copy(self):
        return MultiSet(self._items)

    def discard(self, value, n_copies=1):
        try:
            self._items[value] -= n_copies
            if self._items[value] <= 1:
                del self._items[value]
        except ValueError:
            pass    # discard doesn't raise an exception

    def most_common(self, n=None):
        """
        Return a list of the n most common elements and their counts from the most common to the least. If n is omitted
        or None, most_common() returns all elements in the MultiSet. Elements with equal counts are ordered arbitrarily.

        :return: A list of (item, count) tuples sorted by highest count number
        """
        return self._items.most_common(n)

    def tally(self):
        """
        Returns an iterator of the counts for each element in the MultiSet. The order of iteration is arbitrary.
        """
        for i, count in self._items.items():
            yield (i, count)

    def unique(self):
        """
        Returns the number of unique elements in the MultiSet.

        :return: The count of unique elements
        """
        return len(self.keys())

    def count(self, value):
        """
        Return the number of times the element is in the MultiSet. This method is syntactic sugar for MultiSet[value]

        :param value: the element in the MultiSet
        :return:
        """
        return self._items[value]

    def remove(self, value):
        if value in self:
            if self[value] == 1:
                del self[value]
            else:
                self[value] -= 1
        else:
            raise ValueError('{!r} not in MultiSet'.format(value))

    def __eq__(self, right):
        if not isinstance(right, MultiSet):
            raise TypeError(('Cannot compare MultiSet with object of type {!r}. Can only '
                             'compare MultiSet with MultiSet').format(type(right).__name__))
        else:
            return self._items.__eq__(right._items)

    def __ne__(self, right):
        return not self.__eq__(right)





from functools import cmp_to_key
import unittest
import weakref
import sys

import mochila
from test import test_set_base, test_sequence_base
from test import test_mochila

__author__ = 'Horacio Hoyos'
__copyright__ = 'Copyright , Kinori Technologies'


class OrderedSetTest(test_sequence_base.SequenceTestBase, test_mochila.MochilaTest, test_set_base.SetTestBase, unittest.TestCase):

    type2test = mochila.OrderedSet
    basetype = mochila.OrderedSet

    def test_init(self):
        # Set like init
        s = self.type2test()
        s.__init__(self.word)
        w = []
        for c in self.word:
            if c not in w:
                w.append(c)
        self.assertEqual(s._items, w)
        s.__init__(self.otherword)
        w = []
        for c in self.otherword:
            if c not in w:
                w.append(c)
        self.assertEqual(s._items, w)

        l = self.type2test()
        self.assertRaises(TypeError, l.__init__, (), 2)
        self.assertRaises(TypeError, l.__init__, (), a=12)

    def test_constructor_identity(self):
        s = self.type2test(range(3))
        t = self.type2test(s)
        self.assertNotEqual(id(s), id(t))

    def test_hash(self):
        self.assertRaises(TypeError, hash, self.s)

    def test_clear(self):
        self.s.clear()
        self.assertEqual(self.s, self.type2test())
        self.assertEqual(len(self.s), 0)

    def test_copy(self):
        dup = self.s.copy()
        self.assertEqual(self.s, dup)
        self.assertNotEqual(id(self.s), id(dup))
        self.assertEqual(type(dup), self.basetype)

    def test_add(self):
        self.s.add('Q')
        self.assertIn('Q', self.s)
        dup = self.s.copy()
        self.s.add('Q')
        self.assertEqual(self.s, dup)
        self.assertRaises(TypeError, self.s.add, [])

    def test_remove(self):
        self.s.remove('a')
        self.assertNotIn('a', self.s)
        self.assertRaises(KeyError, self.s.remove, 'Q')
        self.assertRaises(TypeError, self.s.remove, [])

    def test_discard(self):
        self.s.discard('a')
        self.assertNotIn('a', self.s)
        self.s.discard('Q')
        self.assertRaises(TypeError, self.s.discard, [])

    def test_pop(self):
        for i in range(len(self.s)):
            elem = self.s.pop()
            self.assertNotIn(elem, self.s)
        self.assertRaises(KeyError, self.s.pop)

    def test_pop_sequence(self):
        a = self.type2test([-1, 0, 1])
        # Remove random item
        elem = a.pop()
        self.assertFalse(elem in a)
        # Remove from index
        first = a[0]
        elem = a.pop(0)
        self.assertEquals(first, elem)
        self.assertEqual(1, len(a))
        self.assertRaises(IndexError, a.pop, 5)
        a.pop(0)
        self.assertEqual(a, self.type2test())
        self.assertRaises(KeyError, a.pop)
        self.assertRaises(TypeError, a.pop, 42, 42)

    def test_ior(self):
        self.s |= set(self.otherword)
        for c in (self.word + self.otherword):
            self.assertIn(c, self.s)

    def test_iand(self):
        self.s &= set(self.otherword)
        for c in (self.word + self.otherword):
            if c in self.otherword and c in self.word:
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_isub(self):
        self.s -= set(self.otherword)
        for c in (self.word + self.otherword):
            if c in self.word and c not in self.otherword:
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_ixor(self):
        self.s ^= set(self.otherword)
        for c in (self.word + self.otherword):
            if (c in self.word) ^ (c in self.otherword):
                self.assertIn(c, self.s)
            else:
                self.assertNotIn(c, self.s)

    def test_inplace_on_self(self):
        t = self.s.copy()
        t |= t
        self.assertEqual(t, self.s)
        t &= t
        self.assertEqual(t, self.s)
        t -= t
        self.assertEqual(t, self.type2test())
        t = self.s.copy()
        t ^= t
        self.assertEqual(t, self.type2test())

    def test_weakref(self):
        s = self.type2test('gallahad')
        p = weakref.proxy(s)
        self.assertEqual(str(p), str(s))
        s = None
        self.assertRaises(ReferenceError, str, p)

    def test_rich_compare(self):
        class TestRichSetCompare:
            def __gt__(self, some_set):
                self.gt_called = True
                return False

            def __lt__(self, some_set):
                self.lt_called = True
                return False

            def __ge__(self, some_set):
                self.ge_called = True
                return False

            def __le__(self, some_set):
                self.le_called = True
                return False

        # This first tries the builtin rich set comparison, which doesn't know
        # how to handle the custom object. Upon returning NotImplemented, the
        # corresponding comparison on the right object is invoked.
        myset = {1, 2, 3}

        myobj = TestRichSetCompare()
        myset < myobj
        self.assertTrue(myobj.gt_called)

        myobj = TestRichSetCompare()
        myset > myobj
        self.assertTrue(myobj.lt_called)

        myobj = TestRichSetCompare()
        myset <= myobj
        self.assertTrue(myobj.ge_called)

        myobj = TestRichSetCompare()
        myset >= myobj
        self.assertTrue(myobj.le_called)

    @unittest.skipUnless(hasattr(set, "test_c_api"),
                         'C API test only available in a debug build')
    def test_c_api(self):
        self.assertEqual(set().test_c_api(), True)

    def test_and(self):
        # and picks form the other first
        intr = [l for l in self.otherword if l in self.word]
        i = self.type2test(intr)
        self.assertEqual(self.s & self.type2test(self.otherword), i)
        self.assertEqual(self.s & self.otherword, i)

    def test_or(self):
        i = self.type2test([e for s in (self.word, self.otherword) for e in s])
        self.assertEqual(self.s | self.type2test(self.otherword), i)
        self.assertEqual(self.s | self.otherword, i)

    def test_sub(self):
        i = self.type2test([e for e in self.word if e not in self.otherword])
        self.assertEqual(self.s - self.type2test(self.otherword), i)
        self.assertEqual(self.s - self.otherword, i)

    def test_xor(self):
        all = [e for e in self.word if e not in self.otherword]
        all.extend([e for e in self.otherword if e not in self.word])
        i = self.type2test(all)
        self.assertEqual(self.s ^ self.type2test(self.otherword), i)
        self.assertEqual(self.s ^ self.otherword, i)

    def test_contains_fake(self):
        # Set elements need to be hashable
        class AllEq:
            # Sequences must use rich comparison against each item
            # (unless "is" is true, or an earlier item answered)
            # So instances of AllEq must be found in all non-empty sequences.
            def __eq__(self, other):
                return True

            __hash__ = None  # Can't meet hash invariant requirements
        with self.assertRaises(TypeError):
            AllEq() not in self.type2test([])
            AllEq() in self.type2test([1])


    def test_contains_order(self):
        # sets test containment by key
        pass

    def test_count(self):
        # No counting on sets
        pass

    def test_getslice(self):
        l = [0, 1, 2, 3, 4]
        u = self.type2test(l)
        self.assertEqual(u[0:0], self.type2test([]))
        self.assertEqual(u[1:2], self.type2test([1]))
        self.assertEqual(u[-2:-1], self.type2test([3]))
        self.assertEqual(u[-1000:1000], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[1000:-1000], self.type2test([]))
        self.assertEqual(u[:], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[1:None], self.type2test([1, 2, 3, 4]))
        self.assertEqual(u[None:3], self.type2test([0, 1, 2]))

        # Extended slices
        self.assertEqual(u[::], u)
        self.assertEqual(u[::2], self.type2test([0, 2, 4]))
        self.assertEqual(u[1::2], self.type2test([1, 3]))
        self.assertEqual(u[::-1], self.type2test([4, 3, 2, 1, 0]))
        self.assertEqual(u[::-2], self.type2test([4, 2, 0]))
        self.assertEqual(u[3::-2], self.type2test([3, 1]))
        self.assertEqual(u[3:3:-2], self.type2test([]))
        self.assertEqual(u[3:2:-2], self.type2test([3]))
        self.assertEqual(u[3:1:-2], self.type2test([3]))
        self.assertEqual(u[3:0:-2], self.type2test([3, 1]))
        self.assertEqual(u[::-100], self.type2test([4]))
        self.assertEqual(u[100:-100:], self.type2test([]))
        self.assertEqual(u[-100:100:], self.type2test([0, 1, 2, 3, 4]))
        self.assertEqual(u[100:-100:-1], u[::-1])
        self.assertEqual(u[-100:100:-1], self.type2test([]))
        self.assertEqual(u[-100:100:2], self.type2test([0, 2, 4]))

        # Test extreme cases with long ints
        a = self.type2test([0, 1, 2, 3, 4])
        self.assertEqual(a[-pow(2, 128): 3], self.type2test([0, 1, 2]))
        self.assertEqual(a[3: pow(2, 145)], self.type2test([3, 4]))

        # Slice [:] is a copy
        u = self.type2test(l)
        a = u[:]
        a.append(5)
        self.assertFalse(5 in u)

    def test_subscript(self):
        a = self.type2test([10, 11])
        self.assertEqual(a.__getitem__(0), 10)
        self.assertEqual(a.__getitem__(1), 11)
        self.assertEqual(a.__getitem__(-2), 10)
        self.assertEqual(a.__getitem__(-1), 11)
        self.assertRaises(IndexError, a.__getitem__, -3)
        self.assertRaises(IndexError, a.__getitem__, 3)
        self.assertEqual(a.__getitem__(slice(0, 1)), self.type2test([10]))
        self.assertEqual(a.__getitem__(slice(1, 2)), self.type2test([11]))
        self.assertEqual(a.__getitem__(slice(0, 2)), self.type2test([10, 11]))
        self.assertEqual(a.__getitem__(slice(0, 3)), self.type2test([10, 11]))
        self.assertEqual(a.__getitem__(slice(3, 5)), self.type2test([]))
        self.assertRaises(ValueError, a.__getitem__, slice(0, 10, 0))
        self.assertRaises(TypeError, a.__getitem__, 'x')

    def test_iadd(self):
        # No iadd for sets
        pass

    def test_index(self):
        u = self.type2test([0, 1])
        self.assertEqual(u.index(0), 0)
        self.assertEqual(u.index(1), 1)
        self.assertRaises(ValueError, u.index, 2)

        u = self.type2test([-2, -1, 0, 0, 1, 2])
        self.assertEqual(u.index(0), 2)
        self.assertEqual(u.index(0, 2), 2)
        self.assertEqual(u.index(-2, -10), 0)
        self.assertEqual(u.index(2, 0, -10), 4)

        self.assertRaises(TypeError, u.index)

        a = self.type2test([-2, -1, 0, 0, 1, 2])
        self.assertEqual(a.index(0, -4), 2)
        self.assertEqual(a.index(-2, -10), 0)
        self.assertEqual(a.index(0, 2, 4), 2)
        self.assertEqual(a.index(0, -3, -2), 2)
        self.assertEqual(a.index(0, -4 * sys.maxsize, 4 * sys.maxsize), 2)

        a = self.type2test([-2, -1, 0, 1, 2])
        index = a.index([-2, -1])
        self.assertEqual(index, [0, 1])
        index = a.index([1, 0, -2])
        self.assertEqual(index, [3, 2, 0])


    def test_getitem_iterator(self):
        u = self.type2test(['a', 'b', 'c', 'd', 'e'])
        items = [0, 1]
        self.assertEqual(u[items], self.type2test(['a', 'b']))
        items = [2, 4, 0]
        self.assertEqual(u[items], self.type2test(['c', 'e', 'a']))

    def test_setitem(self):
        a = self.type2test([0, 1])
        a[0] = 0
        a[1] = 100
        self.assertEqual(a, self.type2test([0, 100]))
        a[-1] = 200
        self.assertEqual(a, self.type2test([0, 200]))
        a[-2] = 100
        self.assertEqual(a, self.type2test([100, 200]))
        self.assertRaises(IndexError, a.__setitem__, -3, 300)
        self.assertRaises(IndexError, a.__setitem__, 2, 300)

        a = self.type2test([])
        self.assertRaises(IndexError, a.__setitem__, 0, 200)
        self.assertRaises(IndexError, a.__setitem__, -1, 200)
        self.assertRaises(TypeError, a.__setitem__)

        a = self.type2test([0, 1, 2, 3, 4])
        a[0] = 1
        self.assertEqual(a, self.type2test([0, 1, 2, 3, 4]))
        a[2] = 3
        self.assertEqual(a, self.type2test([0, 1, 2, 3, 4]))
        a[0] = 5
        a[1] = 6
        a[2] = 7
        self.assertEqual(a, self.type2test([5, 6, 7, 3, 4]))
        a[-2] = 88
        a[-1] = 99
        self.assertEqual(a, self.type2test([5, 6, 7, 88, 99]))
        a[-2] = 8
        a[-1] = 9
        self.assertEqual(a, self.type2test([5, 6, 7, 8, 9]))

    def test_insert(self):
        a = self.type2test(['a', 'b', 'c'])
        a.insert(0, 'C')
        a.insert(1, 'B')
        a.insert(2, 'A')
        self.assertEqual(a, self.type2test(['C', 'B', 'A', 'a', 'b', 'c']))

        b = a[:]
        b.insert(-2, "foo")
        b.insert(-200, "left")
        b.insert(200, "right")
        self.assertEqual(b, self.type2test(["left", 'C', 'B', 'A', 'a', "foo", 'b', 'c', "right"]))
        self.assertRaises(TypeError, a.insert)

    def test_equals(self):
        self.assertEqual(self.type2test([]), self.type2test())
        s0_3 = self.type2test([0, 1, 2, 3])
        s0_3_bis = self.type2test(list(s0_3))
        self.assertEqual(s0_3, s0_3_bis)
        self.assertTrue(s0_3 is not s0_3_bis)
        self.assertEqual(s0_3, s0_3)
        self.assertFalse(s0_3 == '0123')
        self.assertTrue(self.type2test([]) != '0123')
        self.assertFalse(s0_3 != s0_3)
        s0_4 = self.type2test([0, 1, 2, 3, 4])
        self.assertNotEqual(s0_3, s0_4)
        s1_4 = self.type2test([1, 2, 3, 4])
        self.assertNotEqual(s0_3, s1_4)

    def test_sort(self):
        u = self.type2test([1, 0])
        u.sort()
        self.assertEqual(u, self.type2test([0, 1]))

        u = self.type2test([2, 1, 0, -1, -2])
        u.sort()
        self.assertEqual(u, self.type2test([-2, -1, 0, 1, 2]))

        self.assertRaises(TypeError, u.sort, 42, 42)

        def revcmp(a, b):
            if a == b:
                return 0
            elif a < b:
                return 1
            else:  # a > b
                return -1

        u.sort(key=cmp_to_key(revcmp))
        self.assertEqual(u, self.type2test([2, 1, 0, -1, -2]))

        # The following dumps core in unpatched Python 1.5:
        def myComparison(x, y):
            xmod, ymod = x % 3, y % 7
            if xmod == ymod:
                return 0
            elif xmod < ymod:
                return -1
            else:  # xmod > ymod
                return 1

        z = self.type2test(range(12))
        z.sort(key=cmp_to_key(myComparison))

        self.assertRaises(TypeError, z.sort, 2)

        def selfmodifyingComparison(x, y):
            z.append(12)
            if x == y:
                return 0
            elif x < y:
                return -1
            else:  # x > y
                return 1

        self.assertRaises(ValueError, z.sort,
                          key=cmp_to_key(selfmodifyingComparison))

        self.assertRaises(TypeError, z.sort, 42, 42, 42, 42)
